@@ -2,10 +2,6 @@ setClass("GridTopology",
 	slots = c(cellcentre.offset = "numeric",
 		cellsize = "numeric",
 		cells.dim = "integer"),
-	prototype = list( # not so relevant what this does
-		cellcentre.offset = numeric(0),
-		cellsize = numeric(0), 
-		cells.dim = integer(0)),
 	validity = function(object) {
 		n = length(na.omit(object@cellcentre.offset))
 		if (length(na.omit(object@cellsize)) != n)
