@@ -23,7 +23,7 @@
 ### code chunk number 4: over.Rnw:124-140
 ###################################################
 library(sp)
-x = c(0.5, 0.5, 1.2, 1.5)
+x = c(0.5, 0.5, 1.0, 1.5)
 y = c(1.5, 0.5, 0.5, 0.5)
 xy = cbind(x,y)
 dimnames(xy)[[1]] = c("a", "b", "c", "d")
@@ -34,42 +34,45 @@ ypol = c(0,0,1,1,0)
 pol = SpatialPolygons(list(
 	Polygons(list(Polygon(cbind(xpol-1.05,ypol))), ID="x1"),
 	Polygons(list(Polygon(cbind(xpol,ypol))), ID="x2"),
-	Polygons(list(Polygon(cbind(xpol,ypol-1.05))), ID="x3"),
-	Polygons(list(Polygon(cbind(xpol+1.05,ypol))), ID="x4"),
+	Polygons(list(Polygon(cbind(xpol,ypol - 1.0))), ID="x3"),
+	Polygons(list(Polygon(cbind(xpol + 1.0, ypol))), ID="x4"),
 	Polygons(list(Polygon(cbind(xpol+.4, ypol+.1))), ID="x5")
 	))
 
 
 ###################################################
-### code chunk number 5: over.Rnw:145-150
+### code chunk number 5: over.Rnw:145-153
 ###################################################
-plot(pol, xlim = c(-1.1, 2.1), ylim = c(-1.1, 1.6), border=2:6, axes=TRUE)
+library(RColorBrewer)
+pal = brewer.pal(5, "Set2")
+plot(pol, xlim = c(-1.1, 2.1), ylim = c(-1.1, 1.6), border=pal, axes=TRUE,
+	col = paste0(pal, "4D"))
 points(pts, col='red')
-text(c(-1,0.1,0.1,1.1,0.45), c(0,0,-1.05,0,0.1), 
+text(c(-1,0.1,0.1,1.9,0.45), c(0.05,0.05,-.95,0.05,0.15), 
 	c("x1", "x2", "x3", "x4", "x5"))
 text(coordinates(pts), pos=1, row.names(pts))
 
 
 ###################################################
-### code chunk number 6: over.Rnw:157-158
+### code chunk number 6: over.Rnw:160-161
 ###################################################
 over(pts, pol)
 
 
 ###################################################
-### code chunk number 7: over.Rnw:162-163
+### code chunk number 7: over.Rnw:165-166
 ###################################################
 over(pts, pol, returnList = TRUE)
 
 
 ###################################################
-### code chunk number 8: over.Rnw:166-167
+### code chunk number 8: over.Rnw:169-170
 ###################################################
 pts[pol]
 
 
 ###################################################
-### code chunk number 9: over.Rnw:172-175
+### code chunk number 9: over.Rnw:175-178
 ###################################################
 over(pol, pts)
 over(pol, pts, returnList = TRUE)
@@ -77,7 +80,13 @@ row.names(pol[pts])
 
 
 ###################################################
-### code chunk number 10: over.Rnw:187-196
+### code chunk number 10: over.Rnw:183-184
+###################################################
+over(pol, pol, returnList = TRUE)
+
+
+###################################################
+### code chunk number 11: over.Rnw:203-212
 ###################################################
 zdf = data.frame(z1 = 1:4, z2=4:1, f = c("a", "a", "b", "b"),
 	row.names = c("a", "b", "c", "d"))
@@ -91,32 +100,32 @@ poldf = SpatialPolygonsDataFrame(pol, zpl)
 
 
 ###################################################
-### code chunk number 11: over.Rnw:200-201
+### code chunk number 12: over.Rnw:216-217
 ###################################################
 over(pts, poldf)
 
 
 ###################################################
-### code chunk number 12: over.Rnw:209-210
+### code chunk number 13: over.Rnw:225-226
 ###################################################
 over(pts, poldf[1:2], fn = mean)
 
 
 ###################################################
-### code chunk number 13: over.Rnw:215-216
+### code chunk number 14: over.Rnw:231-232
 ###################################################
 over(pts, poldf, returnList = TRUE)
 
 
 ###################################################
-### code chunk number 14: over.Rnw:220-222
+### code chunk number 15: over.Rnw:236-238
 ###################################################
 over(pol, ptsdf)
 over(pol, ptsdf[1:2], fn = mean)
 
 
 ###################################################
-### code chunk number 15: over.Rnw:240-243
+### code chunk number 16: over.Rnw:256-259
 ###################################################
 l1 = Lines(Line(coordinates(pts)), "L1")
 l2 = Lines(Line(rbind(c(1,1.5), c(1.5,1.5))), "L2")
@@ -124,7 +133,7 @@ L = SpatialLines(list(l1,l2))
 
 
 ###################################################
-### code chunk number 16: over.Rnw:267-271
+### code chunk number 17: over.Rnw:283-287
 ###################################################
 plot(pol, xlim = c(-1.1, 2.1), ylim = c(-1.1, 1.6), border=2:6, axes=TRUE)
 text(c(-1,0.1,0.1,1.1,0.45), c(0,0,-1.05,0,0.1), c("x1", "x2", "x3", "x4", "x5"))
@@ -133,7 +142,7 @@ text(c(0.52, 1.52), c(1.5, 1.5), c("L1", "L2"))
 
 
 ###################################################
-### code chunk number 17: over.Rnw:279-288
+### code chunk number 18: over.Rnw:295-304
 ###################################################
 library(rgeos)
 over(pol, pol)
@@ -147,7 +156,7 @@ over(L, pts)
 
 
 ###################################################
-### code chunk number 18: over.Rnw:293-301
+### code chunk number 19: over.Rnw:309-317
 ###################################################
 data(meuse.grid)
 gridded(meuse.grid) = ~x+y
@@ -160,7 +169,7 @@ lines(sl)
 
 
 ###################################################
-### code chunk number 19: over.Rnw:312-317
+### code chunk number 20: over.Rnw:328-333
 ###################################################
 g = SpatialGrid(GridTopology(c(0,0), c(1,1), c(3,3)))
 p = as(g, "SpatialPolygons")
@@ -170,7 +179,7 @@ text(coordinates(g), labels = 1:9)
 
 
 ###################################################
-### code chunk number 20: over.Rnw:321-325
+### code chunk number 21: over.Rnw:337-341
 ###################################################
 over(g,g)
 over(p,p)
@@ -179,27 +188,27 @@ over(g,p)
 
 
 ###################################################
-### code chunk number 21: over.Rnw:330-332
+### code chunk number 22: over.Rnw:346-348
 ###################################################
 over(px[5], g, returnList = TRUE)
 over(p[c(1,5)], p, returnList = TRUE)
 
 
 ###################################################
-### code chunk number 22: over.Rnw:348-350
+### code chunk number 23: over.Rnw:364-366
 ###################################################
 over(px[5], g, returnList = TRUE, minDimension = 0)
 over(p[c(1,5)], p, returnList = TRUE, minDimension = 0)
 
 
 ###################################################
-### code chunk number 23: over.Rnw:370-371
+### code chunk number 24: over.Rnw:386-387
 ###################################################
 over(p, p, minDimension = 0)
 
 
 ###################################################
-### code chunk number 24: over.Rnw:375-386
+### code chunk number 25: over.Rnw:391-402
 ###################################################
 x2 = x1 = cbind(c(0,1,1,0,0), c(0,0,1,1,0))
 x1[,1] = x1[,1]+0.5
@@ -215,7 +224,7 @@ plot(pt, add=TRUE, col='red', pch=16)
 
 
 ###################################################
-### code chunk number 25: over.Rnw:392-398
+### code chunk number 26: over.Rnw:408-414
 ###################################################
 over(pt,sp)
 over(pt,sp,minDimension=0)
@@ -226,7 +235,7 @@ rgeos::overGeomGeom(pt,sp,returnList=TRUE,minDimension=0)
 
 
 ###################################################
-### code chunk number 26: over.Rnw:414-420
+### code chunk number 27: over.Rnw:430-436
 ###################################################
 over(p[5], p, returnList=TRUE, minDimension=0)
 over(p[5], p, returnList=TRUE, minDimension=1)
@@ -237,33 +246,33 @@ rgeos::overGeomGeom(pt, pt, minDimension=0)
 
 
 ###################################################
-### code chunk number 27: over.Rnw:426-432
+### code chunk number 28: over.Rnw:442-448
 ###################################################
 data(meuse.grid)
 gridded(meuse.grid) = ~x+y
 off = gridparameters(meuse.grid)$cellcentre.offset + 20
 gt = GridTopology(off, c(400,400), c(8,11))
 SG = SpatialGrid(gt)
-agg = aggregate(meuse.grid[3], SG)
+agg = aggregate(meuse.grid[3], SG, mean)
 
 
 ###################################################
-### code chunk number 28: over.Rnw:442-444
+### code chunk number 29: over.Rnw:458-460
 ###################################################
 image(agg)
 points(meuse.grid, pch = 3, cex=.2, col = "#80808080")
 
 
 ###################################################
-### code chunk number 29: over.Rnw:453-456
+### code chunk number 30: over.Rnw:469-472
 ###################################################
-sl.agg = aggregate(meuse.grid[,1:3], sl)
+sl.agg = aggregate(meuse.grid[,1:3], sl, mean)
 class(sl.agg)
 as.data.frame(sl.agg)
 
 
 ###################################################
-### code chunk number 30: over.Rnw:469-478
+### code chunk number 31: over.Rnw:485-494
 ###################################################
 g = SpatialGrid(GridTopology(c(5,5), c(10,10), c(3,3)))
 p = as(g, "SpatialPolygons")
@@ -273,11 +282,11 @@ p$ag1 = aggregate(p, p, mean)[[1]]
 p$ag1a = aggregate(p, p, mean, minDimension = 0)[[1]]
 p$ag2 = aggregate(p, p, mean, minDimension = 1)[[1]]
 p$ag3 = aggregate(p, p, mean, minDimension = 2)[[1]]
-p$ag4 = aggregate(p, p, mean, areaWeighted=TRUE)[[1]]
+p$ag4 = aggregate(p, p, areaWeighted=TRUE)[[1]]
 
 
 ###################################################
-### code chunk number 31: over.Rnw:482-498
+### code chunk number 32: over.Rnw:498-514
 ###################################################
 pts = cbind(c(9,21,21,9,9),c(9,9,21,21,9))
 sq = SpatialPolygons(list(Polygons(list(Polygon(pts)), "ID")))
@@ -298,13 +307,13 @@ spplot(p, names.attr = c("source", "default aggregate", "minDimension=0",
 
 
 ###################################################
-### code chunk number 32: over.Rnw:515-521
+### code chunk number 33: over.Rnw:531-537
 ###################################################
 round(c(
   aggDefault = aggregate(p, sq, mean)[[1]],
   aggMinDim0 = aggregate(p, sq, mean, minDimension = 0)[[1]],
   aggMinDim1 = aggregate(p, sq, mean, minDimension = 1)[[1]],
   aggMinDim2 = aggregate(p, sq, mean, minDimension = 2)[[1]],
-  areaWeighted = aggregate(p, sq, mean, areaWeighted=TRUE)[[1]]), 3)
+  areaWeighted = aggregate(p, sq, areaWeighted=TRUE)[[1]]), 3)
 
 
